import click

from firmitas import __vers__, readconf
from firmitas.base.maintool import gonotify, probedir


@click.command(name="firmitas")
@click.option(
    "-c",
    "--conffile",
    "conffile",
    type=click.Path(exists=True),
    help="Read configuration from the specified Python file",
    default=None,
)
@click.version_option(version=__vers__, prog_name="firmitas")
def main(conffile=None):
    if conffile:
        confdict = {}
        with open(conffile, "r") as confobjc:
            exec(compile(confobjc.read(), conffile, "exec"), confdict)
        readconf(confdict)
    probedir()
    gonotify()



import os
import sys
from datetime import datetime
from pathlib import Path

import yaml
from cryptography import x509
from cryptography.hazmat.backends import default_backend

from firmitas.conf import logrdata, standard
from firmitas.unit import gopagure


def readcert(certobjc):
    commname = certobjc.subject.rfc4514_string().split("=")[1]
    issuauth = certobjc.issuer.rfc4514_string().split("=")[1]
    serialno = certobjc.serial_number
    strtdate, stopdate = certobjc.not_valid_before, certobjc.not_valid_after
    daystobt, daystodd = (strtdate - datetime.now()).days, (stopdate - datetime.now()).days
    cstarted, cstopped = False if daystobt >= 0 else True, False if daystodd >= 0 else True
    logrdata.logrobjc.debug(f"[{commname}] Issued by {issuauth}")
    logrdata.logrobjc.debug(f"[{commname}] Serial number {serialno}")
    logrdata.logrobjc.debug(
        f"[{commname}] Valid from {strtdate} ({abs(daystobt)} days "
        + "{'passed since beginning' if cstarted else 'left before beginning'})"
    )
    logrdata.logrobjc.debug(
        f"[{commname}] Valid until {stopdate} ({abs(daystodd)} days "
        + "{'passed since expiring' if cstopped else 'left before expiring'})"
    )
    return strtdate, stopdate, cstarted, cstopped, daystobt, daystodd, issuauth, serialno


def probedir():
    logrdata.logrobjc.info("Probing into the configured directory")
    doneqant, failqant, totlqant = 0, 0, 0
    if os.path.exists(standard.hostloca):
        standard.certdict = yaml.safe_load(Path(standard.hostloca).read_text())
        logrdata.logrobjc.info(
            f"Validating {len(standard.certdict)} X.509-standard TLS certificates"
        )
        for nameindx in standard.certdict:
            totlqant += 1
            certpath = Path(standard.certloca, standard.certdict[nameindx]["path"])
            if os.path.exists(certpath):
                certobjc = x509.load_pem_x509_certificate(certpath.read_bytes(), default_backend())
                (
                    standard.certdict[nameindx]["certstat"]["strtdate"],
                    standard.certdict[nameindx]["certstat"]["stopdate"],
                    standard.certdict[nameindx]["certstat"]["cstarted"],
                    standard.certdict[nameindx]["certstat"]["cstopped"],
                    standard.certdict[nameindx]["certstat"]["daystobt"],
                    standard.certdict[nameindx]["certstat"]["daystodd"],
                    standard.certdict[nameindx]["certstat"]["issuauth"],
                    standard.certdict[nameindx]["certstat"]["serialno"],
                ) = readcert(certobjc)
                doneqant += 1
                logrdata.logrobjc.info(
                    f"[{nameindx}] The specified X.509-standard TLS certificate was read "
                    + "successfully"
                )
            else:
                failqant += 1
                logrdata.logrobjc.warning(
                    f"[{nameindx}] The specified X.509-standard TLS certificate could not "
                    + "be located"
                )
        logrdata.logrobjc.info(
            f"Of {totlqant} TLS certificates, {doneqant} TLS certificate(s) were read successfully "
            + f"while {failqant} TLS certificate(s) could not be read"
        )
        with open(standard.hostloca, "w") as yamlfile:
            yaml.safe_dump(standard.certdict, yamlfile)
    else:
        logrdata.logrobjc.error(
            "Please set the directory containing the service hostname map properly"
        )
        sys.exit(1)


def gonotify():
    bfstrtcn, afstopcn, totlqant, succqant = 0, 0, 0, 0
    if standard.gitforge == "pagure":
        for certindx in standard.certdict:
            totlqant += 1
            if standard.certdict[certindx]["certstat"]["cstarted"]:
                if standard.certdict[certindx]["certstat"]["cstopped"]:
                    afstopcn += 1
                    logrdata.logrobjc.warning(
                        f"[{certindx}] The specified X.509 TLS certificate is not valid anymore"
                    )
                else:
                    if standard.certdict[certindx]["certstat"]["daystodd"] <= standard.daysqant:
                        logrdata.logrobjc.warning(
                            f"[{certindx}] The specified X.509 TLS certificate is about to expire "
                            + f"in under {standard.daysqant} days from now"
                        )
                        if not standard.certdict[certindx]["notistat"]["done"]:
                            for retcount in range(standard.maxretry):
                                rtrnobjc = gopagure.makenote(
                                    retcount=retcount,
                                    servname=certindx,
                                    strtdate=standard.certdict[certindx]["certstat"]["strtdate"],
                                    stopdate=standard.certdict[certindx]["certstat"]["stopdate"],
                                    daystobt=standard.certdict[certindx]["certstat"]["daystobt"],
                                    daystodd=standard.certdict[certindx]["certstat"]["daystodd"],
                                    certfile=standard.certdict[certindx]["path"],
                                    issuauth=standard.certdict[certindx]["certstat"]["issuauth"],
                                    serialno=standard.certdict[certindx]["certstat"]["serialno"],
                                    assignee=standard.certdict[certindx]["user"],
                                )
                                if rtrnobjc[0]:
                                    succqant += 1
                                    logrdata.logrobjc.info(
                                        f"[{certindx}] The notification ticket for renewing the "
                                        + "TLS certificate has now been created"
                                    )
                                    standard.certdict[certindx]["notistat"]["done"] = rtrnobjc[0]
                                    standard.certdict[certindx]["notistat"]["link"] = rtrnobjc[1]
                                    standard.certdict[certindx]["notistat"]["time"] = rtrnobjc[2]
                                    break
            else:
                bfstrtcn += 1
                logrdata.logrobjc.warning(
                    f"[{certindx}] The specified X.509 TLS certificate is not valid yet"
                )
        logrdata.logrobjc.info(
            f"Of {totlqant} TLS certificates, {bfstrtcn} TLS certificate(s) were not valid "
            + f"yet, {afstopcn} TLS certificates were not valid anymore and {succqant} TLS "
            + "certificates were notified of being near their validity expiry"
        )
        with open(standard.hostloca, "w") as yamlfile:
            yaml.safe_dump(standard.certdict, yamlfile)
    elif standard.gitforge == "gitlab":
        logrdata.logrobjc.error("The notification has not yet been implemented on GitLab")
        sys.exit(1)
    elif standard.gitforge == "github":
        logrdata.logrobjc.error("The notification has not yet been implemented on GitHub")
        sys.exit(1)
    else:
        logrdata.logrobjc.error("The specified ticketing repository forge is not yet supported")
        sys.exit(1)



import os.path
import sys
from importlib import metadata
from logging import getLogger
from logging.config import dictConfig
from pathlib import Path

import yaml

from firmitas.conf import logrdata, standard

__vers__ = metadata.version("firmitas")


def readconf(confobjc):
    standard.gitforge = confobjc.get("gitforge", standard.gitforge)
    standard.repoloca = confobjc.get("repoloca", standard.repoloca)
    standard.reponame = confobjc.get("reponame", standard.reponame)
    standard.username = confobjc.get("username", standard.username)
    standard.password = confobjc.get("password", standard.password)
    standard.daysqant = confobjc.get("daysqant", standard.daysqant)
    standard.maxretry = confobjc.get("maxretry", standard.maxretry)
    standard.certloca = confobjc.get("certloca", standard.certloca)
    standard.hostloca = confobjc.get("hostloca", standard.hostloca)

    dictConfig(standard.logrconf)
    logrdata.logrobjc = getLogger(__name__)

    if standard.gitforge not in ["pagure", "github", "gitlab"]:
        logrdata.logrobjc.error("The specified ticketing repository forge is not yet supported")
        sys.exit(1)

    if not isinstance(standard.daysqant, int):
        logrdata.logrobjc.error(
            "The variable 'daysqant' must have a value of the integer data type only"
        )
        sys.exit(1)
    else:
        if standard.daysqant <= 0:
            logrdata.logrobjc.error(
                "The variable 'daysqant' must have a non-zero positive integer value"
            )
            sys.exit(1)

    if not os.path.exists(standard.certloca):
        logrdata.logrobjc.error(
            "Please set the directory containing X.509 standard TLS certificates properly"
        )
        sys.exit(1)

    if not os.path.exists(standard.hostloca):
        logrdata.logrobjc.error(
            "Please set the directory containing the service hostname map properly"
        )
        sys.exit(1)
    else:
        standard.certdict = yaml.safe_load(Path(standard.hostloca).read_text())



from requests import post

from firmitas.conf import logrdata, standard
from firmitas.unit import issubody, issuhead


def makenote(
    retcount,
    servname,
    strtdate,
    stopdate,
    daystobt,
    daystodd,
    certfile,
    issuauth,
    serialno,
    assignee,
):
    try:
        logrdata.logrobjc.debug(
            f"[{servname}] Notification request attempt count - {retcount+1} of {standard.maxretry}"
        )
        rqstobjc = post(
            url=f"https://pagure.io/api/0/{standard.reponame}/new_issue",
            headers={"Authorization": f"token {standard.password}"},
            data={
                "title": issuhead.format(servname=servname, daysqant=standard.daysqant),
                "issue_content": issubody.format(
                    servname=servname,
                    daysqant=standard.daysqant,
                    strtdate=strtdate,
                    stopdate=stopdate,
                    daystobt=abs(daystobt),
                    daystodd=abs(daystodd),
                    certfile=certfile,
                    issuauth=issuauth,
                    serialno=serialno,
                ),
                "tag": ",".join(standard.tagslist),
                "assignee": assignee,
            },
        )
        logrdata.logrobjc.debug(
            f"[{servname}] The notification request was met with response code "
            + f"{rqstobjc.status_code}"
        )
        if rqstobjc.status_code == 200:
            logrdata.logrobjc.debug(
                f"[{servname}] The created notification ticket was created with ID "
                + f"#{rqstobjc.json()['issue']['id']} ({rqstobjc.json()['issue']['full_url']})."
            )
            return (
                True,
                rqstobjc.json()["issue"]["full_url"],
                rqstobjc.json()["issue"]["date_created"],
            )
        else:
            return False, "", ""
    except Exception as expt:
        logrdata.logrobjc.error(
            f"[{servname}] The notification ticket could not be created - {expt}"
        )
        return False, "", ""



issuhead = "[FMTS] TLS certificate for {servname} service is about to expire in {daysqant} days"

issubody = """
This is to inform that the TLS certificate for **{servname}** service will expire in about **{daysqant} day(s)** from now on **{stopdate} UTC**. The following are information relevant to the associated TLS certificate.

- **Service name** - **{servname}** (Certificate stored as **{certfile}**)
- **Issuing authority** - {issuauth} (**#{serialno}**)
- **Validity starting** - **{strtdate} UTC** (**{daystobt} day(s)** passed since beginning)
- **Validity ending** - **{stopdate} UTC** (**{daystodd} day(s)** left before expiring)

The point of contact for the service have been tagged into this ticket and notified about the same. It is strongly recommended to promptly renew the TLS certificate for the service before the existing one expires.

_This issue ticket was automatically created by the [**Firmitas notification service**](https://gitlab.com/t0xic0der/firmitas). Please contact [**Fedora Infrastructure**](https://pagure.io/fedora-infrastructure/issues) team if you believe that this notification is mistaken._
"""  # noqa



"""
Variables related to notifications
"""

# The source code forge on which the issue tickets need to be created
gitforge = "pagure"

# The location of the ticketing repository
repoloca = ""

# The name of the ticketing repository with namespace
reponame = ""

# The username to masquerade as in order to create notification tickets
username = ""

# The API key for the source code forge pertaining to the user
password = ""

# Number of days from validity expiry to make the notification for
daysqant = 30

# List of labels to tag the notification tickets with
tagslist = ["firmitas", "automate", "notifier"]

# Maximum number of retries to opening the notification ticket
maxretry = 5

"""
Variables related to probing
"""

# The location of the X.509 standard TLS certificates
certloca = "/var/tmp/firmitas/certhere"

# The location of the service hostnames and maintainers map
hostloca = "/var/tmp/firmitas/certlist.yml"

"""
Variables related to logging
"""

# The default configuration for service logging
logrconf = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "standard": {
            "format": "[FMTS] %(asctime)s [%(levelname)s] %(message)s",
            "datefmt": "[%Y-%m-%d %I:%M:%S %z]",
        },
    },
    "handlers": {
        "console": {
            "level": "DEBUG",
            "formatter": "standard",
            "class": "logging.StreamHandler",
            "stream": "ext://sys.stdout",
        },
    },
    "root": {
        "level": "DEBUG",
        "handlers": ["console"],
    },
}

"""
Variables used for computing
"""

certdict = {}



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['firmitas', 'firmitas.base', 'firmitas.conf', 'firmitas.unit']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.0,<9',
 'cryptography>=36.0.0,<40',
 'pyyaml>=5.0.0,<7',
 'requests>=2.20.0,<3']

entry_points = \
{'console_scripts': ['firmitas = firmitas.main:main']}

setup_kwargs = {
    'name': 'firmitas',
    'version': '0.1.1',
    'description': 'Simple notification service for X.509-standard TLS certificate statuses',
    'long_description': '# firmitas\n\nSimple notification service for X.509-standard TLS certificate statuses\n\n## `__usage__`\n\n1. Clone the repository to your local storage and make it the present working directory.\n   ```\n   $ git clone https://gitlab.com/t0xic0der/firmitas.git\n   ```\n   ```\n   $ cd firmitas\n   ```\n2. Ensure that [`Poetry`](https://python-poetry.org/) and [Virtualenv](https://pypi.org/project/virtualenv/) are installed and up-to-date on the system.\n   ```\n   $ sudo dnf install --assumeyes poetry virtualenv\n   ```\n3. Create a virtual environment within the cloned local directory and enable it.\n   ```\n   $ virtualenv venv\n   ```\n   ```\n   $ source venv/bin/activate\n   ```\n4. Install the project and its dependencies in the enabled virtual environment.\n   ```\n   $ (venv) poetry install\n   ```\n5. Make a copy of the project configuration file and make your changes to it.\n   ```\n   $ (venv) cp firmitas/conf/standard.py firmitas/conf/myconfig.py\n   ```\n   The project configuration file houses the following variables that can be modified according to the requirements of the project user.  \n   - `gitforge` - The source code forge on which the issue tickets need to be created. The valid options for this configuration variable are `github`, `gitlab` and `pagure`. The support for `pagure` is implemented while that for `github` and `gitlab` is planned.\n   - `repoloca` - The location of the ticketing repository. This is required for source code forges that require the absolute location of the remote ticketing repository for the project to be able to access the API and create notification tickets.\n   - `reponame` - The name of the ticketing repository. This is required for source code forges that require the repository name with namespace of the remote ticketing repository for the project to be able to access the API and create notification tickets.\n   - `username` - The username to masquerade as in order to create notification tickets. This is required for source code forges that require the username with whom an API token is associated for the project to be authenticated on their behalf.\n   - `password` - The password or API token belonging to the aforementioned username. This is required for source code forges that require the password or API token associated with the ticketing repository to authenticate the project on the owner\'s behalf.\n   - `daysqant` - The minimum number of days remaining from the validity expiry date to make notifications for. As a sane default, it is set to open up a notification ticket when the TLS certificate is 30 days away from its validation expiry date.\n   - `tagslist` - The list of labels to the tag the notification tickets with. A minimum of one label is required to ensure that there is a way to filter out the automated notification tickets on a shared issue tracker from the manually created ones.\n   - `maxretry` - The maximum number of retries to make when the process of opening up a notification ticket fails. As a sane default, it is set to allow up to 5 retries, and it is a good practice to have a value greater than one to compensate for spotty connections.\n   - `certloca` - The location where the X.509 standard TLS certificates are stored. Note that this refers to a locally available storage location and not a remotely available storage location. The default is set as "/var/tmp/firmitas/certhere" directory.\n   - `hostloca` - The location where the mapping file of service hostnames, maintainers, certificate statistics and notification statistics are stored. As with the previous configuration variable, even this one refers to a locally available storage location.\n   - `logrconf` - The configuration variable that sets the logging behaviour for the project. As a sane default, the logging level has been set to "DEBUG" to allow for greater verbosity in details and a custom format to the console handler.\n   - `certdict` - The global variable used across the project to share the details of the certificates to be probed into, the statistics of issuing authority, serial number, dates information and much more. Do not change it as this gets overridden.\n6. Make a copy of the mapping configuration file and make your changes to it.\n   ```\n   $ (venv) cp firmitas/conf/certlist.yml firmitas/conf/mytlscts.yml\n   ```\n   The mapping configuration file houses a list of service hostnames having the following variables that can be either modified according to the requirements of the project user or computed by the project during its runtime.\n   - `path`: The location of the X.509 standard TLS certificate file relative to the "certloca" variable previously set in the project configuration file. This helps the project to locate the X.509 standard TLS certificate file is read and acted upon.\n   - `user`: The username on the source code forge that was previously set on the "gitforge" variable in the project configuration file. If the username is not available on the stated source code forge, the notification ticket making process will error out.\n   - `certstat`: This consists of a list of variables that must not be set manually as they would be overridden by the project during its runtime. Here is a list of those variables with their associated meanings and significance.\n     - `cstarted`: A variable of boolean type. This is computed as TRUE if the current datetime is greater than the "not valid before" datetime of the stated X.509 standard TLS certificate and FALSE if the current datetime is lesser than the "not valid before" datetime of the same.\n     - `cstopped`: A variable of boolean type. This is computed as TRUE if the current datetime is greater than the "not valid after" datetime of the stated X.509 standard TLS certificate and FALSE if the current datetime is less than the "not valid after" datetime of the same.\n     - `daystobt`: A variable of integer type. This is computed as the difference in the number of days from the current datetime to the datetime from when the stated X.509 standard TLS certificate becomes valid. This can either be a positive integer if the "not valid before" datetime has not been reached or a negative integer if the "not valid before" datetime has been passed.\n     - `daystodd`: A variable of integer type. This is computed as the difference in the number of days from the current datetime to the datetime to when the stated X.509 standard TLS certificate becomes expired. This can either be a positive integer if the "not valid after" datetime has not been reached or a negative integer if the "not valid after" datetime has been passed.\n     - `issuauth`: A variable of string type. This stores the found name of the issuing authority for the stated X.509 standard TLS certificate. This can be useful if the same issuing authority is planned to be used to regenerate a new one from.\n     - `serialno`: A variable of string type. This stores the found serial number of the stated X.509 standard TLS certificate. This can be useful to de-validate the existing certificate before opting in to regenerate a new one.\n     - `strtdate`: A variable of datetime type. This is computed as the datetime data consisting of the "not valid before" datetime value.\n     - `stopdate`: A variable of datetime type. This is computed as the datetime data consisting of the "not valid after" datetime value.\n   - `notistat`: This consists of a list of variables that must not be set manually as they would be overridden by the project during its runtime. Here is a list of those variables with their associated meanings and significance.\n     - `done`: A variable of boolean type. This stores the flag to state if the notification about the expiry of the stated X.509 standard TLS certificate has been made. The variable is set to TRUE if the notification has been created and FALSE otherwise.\n     - `link`: A variable of string type. This stores the location of the notification ticket on the selected source code forge that was previously set on the "gitforge" variable in the project configuration file.\n     - `time`: A variable of datetime type. This stores the datetime information of when the previously stated notification ticket was created. This can be useful to track down if there are any repeated notifications made.\n7. Make sure that the location of the custom mapping configuration file is pointed correctly at in the custom project configuration file.\n8. View the console help menu of the project service by running the following command.\n   ```\n   $ (venv) firmitas --help\n   ```\n   Output\n   ```\n   Usage: firmitas [OPTIONS]\n   \n   Options:\n     -c, --conffile PATH  Read configuration from the specified Python file\n     --version            Show the version and exit.\n     --help               Show this message and exit.\n   ```\n9. With the configuration variables set appropriately, run the project service by executing the following command.\n   ```\n   $ (venv) firmitas --conffile firmitas/conf/myconfig.py\n   ```\n   Refrain from making any changes in the existing entries of the mapping configuration file after the first successful run unless it is absolutely necessary to do so as the project service writes and references notification creation status from that file. Any unmonitored change to the existing entries in said file after the first successful run of the project service could lead to unintended consequences such as duplicate notification entries, untracked notification tickets pertaining to services etc.\n\n## `__license__`\n\nThis project is licensed under GNU General Public License 3.0 or later.\n',
    'author': 'Akashdeep Dhar',
    'author_email': 'akashdeep.dhar@gmail.com',
    'maintainer': 'Akashdeep Dhar',
    'maintainer_email': 'akashdeep.dhar@gmail.com',
    'url': 'https://gitlab.com/t0xic0der/firmitas/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4',
}


setup(**setup_kwargs)



import logging

logging.basicConfig(
    format="[FMTS] %(asctime)s [%(levelname)s] %(message)s",
    datefmt="[%Y-%m-%d %I:%M:%S %z]",
    level=logging.INFO,
)


logrobjc = logging.getLogger(__name__)
